@@ -78,7 +78,6 @@ def test_dashboard(loop):
     with popen(
         ["dask", "scheduler", "--host", f"127.0.0.1:{port}"],
     ):
-
         with Client(f"127.0.0.1:{port}", loop=loop) as c:
             dashboard_port = _get_dashboard_port(c)
 
@@ -100,7 +99,7 @@ def test_dashboard(loop):
                     raise
                 sleep(0.1)
 
-    with pytest.raises(Exception):
+    with pytest.raises(requests.ConnectionError):
         requests.get(f"http://127.0.0.1:{dashboard_port}/status/")
 
 
@@ -128,7 +127,8 @@ def test_dashboard_non_standard_ports(loop):
             except Exception:
                 sleep(0.1)
                 assert time() < start + 20
-    with pytest.raises(Exception):
+
+    with pytest.raises(requests.ConnectionError):
         requests.get(f"http://localhost:{port2}/status/")
 
 
@@ -152,7 +152,7 @@ def test_multiple_protocols(loop):
 @pytest.mark.skipif(not LINUX, reason="Need 127.0.0.2 to mean localhost")
 def test_dashboard_allowlist(loop):
     pytest.importorskip("bokeh")
-    with pytest.raises(Exception):
+    with pytest.raises(requests.ConnectionError):
         requests.get("http://localhost:8787/status/").ok
 
     port = open_port()
@@ -596,7 +596,7 @@ def test_signal_handling(loop, sig):
 
 
 @pytest.mark.skipif(WINDOWS, reason="POSIX only")
-def test_deprecated_single_executable(loop):
+def test_single_executable_deprecated(loop):
     port = open_port()
     with popen(
         [
