@@ -25,10 +25,12 @@ from distributed.comm import (
     unparse_host_port,
 )
 from distributed.comm.registry import backends, get_backend
-from distributed.compatibility import to_thread
+from distributed.comm.tcp import get_stream_address
+from distributed.compatibility import asyncio_run
+from distributed.config import get_loop_factory
 from distributed.metrics import time
 from distributed.protocol import Serialized, deserialize, serialize, to_serialize
-from distributed.utils import get_ip, get_ipv6, get_mp_context
+from distributed.utils import get_ip, get_ipv6, get_mp_context, wait_for
 from distributed.utils_test import (
     gen_test,
     get_cert,
@@ -437,9 +439,11 @@ async def run_coro(func, *args, **kwargs):
 async def run_coro_in_thread(func, *args, **kwargs):
     async def run_with_timeout():
         t = asyncio.create_task(func(*args, **kwargs))
-        return await asyncio.wait_for(t, timeout=10)
+        return await wait_for(t, timeout=10)
 
-    return await to_thread(asyncio.run, run_with_timeout())
+    return await asyncio.to_thread(
+        asyncio_run, run_with_timeout(), loop_factory=get_loop_factory()
+    )
 
 
 @gen_test()
@@ -755,7 +759,6 @@ async def check_comm_closed_implicit(
         await comm.close()
 
     async with listen(addr, handle_comm, **listen_args) as listener:
-
         comm = await connect(listener.contact_address, **connect_args)
         with pytest.raises(CommClosedError):
             await comm.write({})
@@ -909,7 +912,7 @@ async def test_comm_closed_on_read_error(tcp):
     reader, writer = await get_tcp_comm_pair()
 
     with pytest.raises(asyncio.TimeoutError):
-        await asyncio.wait_for(reader.read(), 0.01)
+        await wait_for(reader.read(), 0.01)
 
     assert reader.closed()
 
@@ -936,7 +939,6 @@ async def test_retry_connect(tcp, monkeypatch):
 
     class UnreliableConnector(tcp.TCPConnector):
         def __init__(self):
-
             self.num_failures = 2
             self.failures = 0
             super().__init__()
@@ -964,6 +966,7 @@ async def test_retry_connect(tcp, monkeypatch):
         listener.stop()
 
 
+@pytest.mark.slow
 @gen_test()
 async def test_handshake_slow_comm(tcp, monkeypatch):
     class SlowComm(tcp.TCP):
@@ -998,11 +1001,9 @@ async def test_handshake_slow_comm(tcp, monkeypatch):
 
         import dask
 
-        with dask.config.set({"distributed.comm.timeouts.connect": "100ms"}):
-            with pytest.raises(
-                IOError, match="Timed out during handshake while connecting to"
-            ):
-                await connect(listener.contact_address)
+        # The connect itself is fast. Only the handshake is slow
+        with dask.config.set({"distributed.comm.timeouts.connect": "500ms"}):
+            await connect(listener.contact_address)
     finally:
         listener.stop()
 
@@ -1342,6 +1343,15 @@ async def test_tcp_adresses(tcp):
     await check_addresses(a, b)
 
 
+@gen_test()
+async def test_get_stream_address_raises_if_closed():
+    a, b = await get_tcp_comm_pair()
+    a.abort()
+    with pytest.raises(OSError):
+        get_stream_address(a)
+    b.abort()
+
+
 @gen_test()
 async def test_tls_adresses(tcp):
     a, b = await get_tls_comm_pair()
