@@ -13,6 +13,8 @@ from distributed.utils_test import (
     BlockedGetData,
     _LockedCommPool,
     assert_story,
+    async_poll_for,
+    freeze_batched_send,
     gen_cluster,
     inc,
     lock_inc,
@@ -22,7 +24,6 @@ from distributed.utils_test import (
 from distributed.worker_state_machine import (
     AddKeysMsg,
     ComputeTaskEvent,
-    DigestMetric,
     Execute,
     ExecuteFailureEvent,
     ExecuteSuccessEvent,
@@ -32,6 +33,8 @@ from distributed.worker_state_machine import (
     GatherDepNetworkFailureEvent,
     GatherDepSuccessEvent,
     LongRunningMsg,
+    ReleaseWorkerDataMsg,
+    RemoveReplicasEvent,
     RescheduleEvent,
     SecedeEvent,
     TaskFinishedMsg,
@@ -146,12 +149,11 @@ async def test_worker_stream_died_during_comm(c, s, a, b):
     write_event.set()
 
     await res
-    assert any("receive-dep-failed" in msg for msg in b.state.log)
+    assert any("gather-dep-failed" in msg for msg in b.state.log)
 
 
 @gen_cluster(client=True, nthreads=[("", 1)])
 async def test_flight_to_executing_via_cancelled_resumed(c, s, b):
-
     block_get_data = asyncio.Lock()
     block_compute = Lock()
     enter_get_data = asyncio.Event()
@@ -192,7 +194,7 @@ async def test_flight_to_executing_via_cancelled_resumed(c, s, b):
         assert await fut2 == 3
 
         b_story = b.state.story(fut1.key)
-        assert any("receive-dep-failed" in msg for msg in b_story)
+        assert any("gather-dep-failed" in msg for msg in b_story)
         assert any("cancelled" in msg for msg in b_story)
         assert any("resumed" in msg for msg in b_story)
 
@@ -265,19 +267,8 @@ def test_flight_cancelled_error(ws):
 
 @gen_cluster(client=True, nthreads=[("", 1)])
 async def test_in_flight_lost_after_resumed(c, s, b):
-    lock_executing = Lock()
-
-    def block_execution(lock):
-        lock.acquire()
-        return 1
-
     async with BlockedGetData(s.address) as a:
-        fut1 = c.submit(
-            block_execution,
-            lock_executing,
-            workers=[a.address],
-            key="fut1",
-        )
+        fut1 = c.submit(inc, 1, workers=[a.address], key="fut1")
         # Ensure fut1 is in memory but block any further execution afterwards to
         # ensure we control when the recomputation happens
         await wait(fut1)
@@ -286,7 +277,6 @@ async def test_in_flight_lost_after_resumed(c, s, b):
         # This ensures that B already fetches the task, i.e. after this the task
         # is guaranteed to be in flight
         await a.in_get_data.wait()
-        assert fut1.key in b.state.tasks
         assert b.state.tasks[fut1.key].state == "flight"
 
         s.set_restrictions({fut1.key: [a.address, b.address]})
@@ -294,30 +284,12 @@ async def test_in_flight_lost_after_resumed(c, s, b):
         # to be recomputed on B
         await s.remove_worker(a.address, stimulus_id="foo", close=False, safe=True)
 
-        while not b.state.tasks[fut1.key].state == "resumed":
-            await asyncio.sleep(0.01)
+        await wait_for_state(fut1.key, "resumed", b, interval=0)
 
         fut1.release()
         fut2.release()
 
-        while not b.state.tasks[fut1.key].state == "cancelled":
-            await asyncio.sleep(0.01)
-
-        a.block_get_data.set()
-        while b.state.tasks:
-            await asyncio.sleep(0.01)
-
-    assert_story(
-        b.state.story(fut1.key),
-        expect=[
-            # The initial free-keys is rejected
-            ("free-keys", (fut1.key,)),
-            (fut1.key, "resumed", "released", "cancelled", {}),
-            # After gather_dep receives the data, the task is forgotten
-            (fut1.key, "cancelled", "memory", "released", {fut1.key: "forgotten"}),
-            (fut1.key, "released", "forgotten", "forgotten", {}),
-        ],
-    )
+        await async_poll_for(lambda: not b.state.tasks, timeout=5)
 
 
 @gen_cluster(client=True)
@@ -457,7 +429,7 @@ async def test_resumed_cancelled_handle_compute(
     Given the history of a task
     executing -> cancelled -> resumed(fetch)
 
-    A handle_compute should properly restore executing.
+    the scheduler should reject the result upon completion and reschedule the task.
     """
     # This test is heavily using set_restrictions to simulate certain scheduler
     # decisions of placing keys
@@ -517,6 +489,8 @@ async def test_resumed_cancelled_handle_compute(
         await lock_compute.release()
         await exit_compute.wait()
 
+        await async_poll_for(lambda: f3.key not in b.state.tasks, timeout=5)
+
     f1 = c.submit(inc, 1, key="f1", workers=[a.address])
     f2 = c.submit(inc, f1, key="f2", workers=[a.address])
     f3 = c.submit(inc, f2, key="f3", workers=[b.address])
@@ -535,7 +509,17 @@ async def test_resumed_cancelled_handle_compute(
                 (f3.key, "ready", "executing", "executing", {}),
                 (f3.key, "executing", "released", "cancelled", {}),
                 (f3.key, "cancelled", "fetch", "resumed", {}),
-                (f3.key, "resumed", "memory", "memory", {}),
+                (f3.key, "resumed", "released", "cancelled", {}),
+                (
+                    f3.key,
+                    "cancelled",
+                    "memory",
+                    "released",
+                    {f2.key: "released", f3.key: "forgotten"},
+                ),
+                (f3.key, "released", "forgotten", "forgotten", {f2.key: "forgotten"}),
+                (f3.key, "ready", "executing", "executing", {}),
+                (f3.key, "executing", "memory", "memory", {}),
             ],
         )
 
@@ -548,11 +532,15 @@ async def test_resumed_cancelled_handle_compute(
                 (f3.key, "ready", "executing", "executing", {}),
                 (f3.key, "executing", "released", "cancelled", {}),
                 (f3.key, "cancelled", "fetch", "resumed", {}),
-                (f3.key, "resumed", "error", "released", {f3.key: "fetch"}),
-                (f3.key, "fetch", "flight", "flight", {}),
-                (f3.key, "flight", "missing", "missing", {}),
-                (f3.key, "missing", "waiting", "waiting", {f2.key: "fetch"}),
-                (f3.key, "waiting", "ready", "ready", {f3.key: "executing"}),
+                (f3.key, "resumed", "released", "cancelled", {}),
+                (
+                    f3.key,
+                    "cancelled",
+                    "error",
+                    "released",
+                    {f2.key: "released", f3.key: "forgotten"},
+                ),
+                (f3.key, "released", "forgotten", "forgotten", {f2.key: "forgotten"}),
                 (f3.key, "ready", "executing", "executing", {}),
                 (f3.key, "executing", "memory", "memory", {}),
             ],
@@ -567,14 +555,24 @@ async def test_resumed_cancelled_handle_compute(
                 (f3.key, "ready", "executing", "executing", {}),
                 (f3.key, "executing", "released", "cancelled", {}),
                 (f3.key, "cancelled", "fetch", "resumed", {}),
-                (f3.key, "resumed", "waiting", "executing", {}),
+                (f3.key, "resumed", "released", "cancelled", {}),
+                (f3.key, "cancelled", "waiting", "executing", {}),
+                (f3.key, "executing", "memory", "memory", {}),
+                (
+                    f3.key,
+                    "memory",
+                    "released",
+                    "released",
+                    {f2.key: "released", f3.key: "forgotten"},
+                ),
+                (f3.key, "released", "forgotten", "forgotten", {f2.key: "forgotten"}),
+                (f3.key, "ready", "executing", "executing", {}),
                 (f3.key, "executing", "memory", "memory", {}),
             ],
         )
 
     elif wait_for_processing and raise_error:
-        with pytest.raises(RuntimeError, match="test error"):
-            await f3
+        assert await f4 == 4 + 2
 
         assert_story(
             b.state.story(f3.key),
@@ -582,14 +580,242 @@ async def test_resumed_cancelled_handle_compute(
                 (f3.key, "ready", "executing", "executing", {}),
                 (f3.key, "executing", "released", "cancelled", {}),
                 (f3.key, "cancelled", "fetch", "resumed", {}),
-                (f3.key, "resumed", "waiting", "executing", {}),
+                (f3.key, "resumed", "released", "cancelled", {}),
+                (f3.key, "cancelled", "waiting", "executing", {}),
                 (f3.key, "executing", "error", "error", {}),
+                (
+                    f3.key,
+                    "error",
+                    "released",
+                    "released",
+                    {f2.key: "released", f3.key: "forgotten"},
+                ),
+                (f3.key, "released", "forgotten", "forgotten", {f2.key: "forgotten"}),
+                (f3.key, "ready", "executing", "executing", {}),
+                (f3.key, "executing", "memory", "memory", {}),
             ],
         )
     else:
         assert False, "unreachable"
 
 
+@pytest.mark.parametrize("raise_error", [True, False])
+@gen_cluster(client=True)
+async def test_cancelled_handle_compute(c, s, a, b, raise_error):
+    """
+    Given the history of a task
+    executing -> cancelled
+
+    A handle_compute should cause the result of the cancelled task to be rejected
+    by the scheduler and the task to be re-run.
+
+    See Also
+    --------
+    test_resumed_cancelled_handle_compute
+    """
+    # This test is heavily using set_restrictions to simulate certain scheduler
+    # decisions of placing keys
+
+    lock_compute = Lock()
+    await lock_compute.acquire()
+    enter_compute = Event()
+    exit_compute = Event()
+
+    def block(x, lock, enter_event, exit_event):
+        enter_event.set()
+        with lock:
+            if raise_error:
+                raise RuntimeError("test error")
+            return x + 1
+
+    f1 = c.submit(inc, 1, key="f1", workers=[a.address])
+    f2 = c.submit(inc, f1, key="f2", workers=[a.address])
+    f3 = c.submit(
+        block,
+        f2,
+        lock=lock_compute,
+        enter_event=enter_compute,
+        exit_event=exit_compute,
+        key="f3",
+        workers=[b.address],
+    )
+
+    f4 = c.submit(sum, [f1, f3], key="f4", workers=[b.address])
+
+    await enter_compute.wait()
+
+    async def release_all_futures():
+        futs = [f1, f2, f3, f4]
+        for fut in futs:
+            fut.release()
+
+        while any(fut.key in s.tasks for fut in futs):
+            await asyncio.sleep(0.05)
+
+    await release_all_futures()
+    await wait_for_state(f3.key, "cancelled", b)
+
+    f1 = c.submit(inc, 1, key="f1", workers=[a.address])
+    f2 = c.submit(inc, f1, key="f2", workers=[a.address])
+    f3 = c.submit(inc, f2, key="f3", workers=[b.address])
+    f4 = c.submit(sum, [f1, f3], key="f4", workers=[b.address])
+
+    await wait_for_state(f3.key, "processing", s)
+    await lock_compute.release()
+
+    assert await f4 == 4 + 2
+
+    if raise_error:
+        assert_story(
+            b.state.story(f3.key),
+            expect=[
+                (f3.key, "ready", "executing", "executing", {}),
+                (f3.key, "executing", "released", "cancelled", {}),
+                (f3.key, "cancelled", "waiting", "executing", {}),
+                (f3.key, "executing", "error", "error", {}),
+                (
+                    f3.key,
+                    "error",
+                    "released",
+                    "released",
+                    {f2.key: "released", f3.key: "forgotten"},
+                ),
+                (f3.key, "released", "forgotten", "forgotten", {f2.key: "forgotten"}),
+                (f3.key, "ready", "executing", "executing", {}),
+                (f3.key, "executing", "memory", "memory", {}),
+            ],
+        )
+    else:
+        assert_story(
+            b.state.story(f3.key),
+            expect=[
+                (f3.key, "ready", "executing", "executing", {}),
+                (f3.key, "executing", "released", "cancelled", {}),
+                (f3.key, "cancelled", "waiting", "executing", {}),
+                (f3.key, "executing", "memory", "memory", {}),
+                (
+                    f3.key,
+                    "memory",
+                    "released",
+                    "released",
+                    {f2.key: "released", f3.key: "forgotten"},
+                ),
+                (f3.key, "released", "forgotten", "forgotten", {f2.key: "forgotten"}),
+                (f3.key, "ready", "executing", "executing", {}),
+                (f3.key, "executing", "memory", "memory", {}),
+            ],
+        )
+
+
+@gen_cluster(client=True)
+async def test_cancelled_task_error_rejected(c, s, a, b):
+    """
+    Given the history of a task
+    executing -> cancelled
+
+    An error in the cancelled task is rejected by the scheduler and superseded
+    by a more recent run on another worker.
+
+    """
+    # This test is heavily using set_restrictions to simulate certain scheduler
+    # decisions of placing keys
+
+    lock_erring = Lock()
+    enter_compute_erring = Event()
+    exit_compute_erring = Event()
+    lock_successful = Lock()
+    enter_compute_successful = Event()
+    exit_compute_successful = Event()
+
+    await lock_erring.acquire()
+    await lock_successful.acquire()
+
+    def block(x, lock, enter_event, exit_event, raise_error):
+        enter_event.set()
+        try:
+            with lock:
+                if raise_error:
+                    raise RuntimeError("test_error")
+                return x + 1
+        finally:
+            exit_event.set()
+
+    f1 = c.submit(inc, 1, key="f1", workers=[a.address])
+    f2 = c.submit(inc, f1, key="f2", workers=[a.address])
+    f3 = c.submit(
+        block,
+        f2,
+        lock=lock_erring,
+        enter_event=enter_compute_erring,
+        exit_event=exit_compute_erring,
+        raise_error=True,
+        key="f3",
+        workers=[b.address],
+    )
+
+    f4 = c.submit(sum, [f1, f3], key="f4", workers=[b.address])
+
+    await enter_compute_erring.wait()
+
+    async def release_all_futures():
+        futs = [f1, f2, f3, f4]
+        for fut in futs:
+            fut.release()
+
+        while any(fut.key in s.tasks for fut in futs):
+            await asyncio.sleep(0.05)
+
+    with freeze_batched_send(s.stream_comms[b.address]):
+        await release_all_futures()
+
+        f1 = c.submit(inc, 1, key="f1", workers=[a.address])
+        f2 = c.submit(inc, f1, key="f2", workers=[a.address])
+        f3 = c.submit(
+            block,
+            f2,
+            lock=lock_successful,
+            enter_event=enter_compute_successful,
+            exit_event=exit_compute_successful,
+            raise_error=False,
+            key="f3",
+            workers=[a.address],
+        )
+        f4 = c.submit(sum, [f1, f3], key="f4", workers=[b.address])
+
+        await wait_for_state(f3.key, "processing", s)
+        await enter_compute_successful.wait()
+
+        await lock_erring.release()
+        await wait_for_state(f3.key, "error", b)
+
+    await lock_successful.release()
+    assert await f4 == 4 + 2
+
+    assert_story(
+        b.state.story(f3.key),
+        expect=[
+            (f3.key, "ready", "executing", "executing", {}),
+            (f3.key, "executing", "error", "error", {}),
+            (
+                f3.key,
+                "error",
+                "released",
+                "released",
+                {f3.key: "forgotten"},
+            ),
+            (f3.key, "released", "forgotten", "forgotten", {}),
+        ],
+    )
+
+    assert_story(
+        a.state.story(f3.key),
+        expect=[
+            (f3.key, "ready", "executing", "executing", {}),
+            (f3.key, "executing", "memory", "memory", {}),
+        ],
+    )
+
+
 @pytest.mark.parametrize("intermediate_state", ["resumed", "cancelled"])
 @pytest.mark.parametrize("close_worker", [False, True])
 @gen_cluster(client=True, config={"distributed.comm.timeouts.connect": "500ms"})
@@ -691,7 +917,6 @@ def test_workerstate_executing_skips_fetch_on_success(ws_with_running_task):
         ExecuteSuccessEvent.dummy("x", 123, stimulus_id="s3"),
     )
     assert instructions == [
-        DigestMetric(name="compute-duration", value=1.0, stimulus_id="s3"),
         AddKeysMsg(keys=["x"], stimulus_id="s3"),
         Execute(key="y", stimulus_id="s3"),
     ]
@@ -706,7 +931,7 @@ def test_workerstate_executing_failure_to_fetch(ws_with_running_task):
     - executing -> long-running -> cancelled -> resumed(fetch)
 
     The task execution later terminates with a failure.
-    This is an edge case interaction between work stealing and a task that does not
+    This is an edge case interaction involving task cancellation and a task that does not
     deterministically succeed or fail when run multiple times or on different workers.
 
     Test that the task is fetched from the other worker. This is to avoid having to deal
@@ -781,7 +1006,6 @@ def test_workerstate_flight_failure_to_executing(ws, block_queue):
         )
         assert instructions == [
             Execute(key="z", stimulus_id="s4"),
-            DigestMetric(name="compute-duration", value=1.0, stimulus_id="s6"),
             TaskFinishedMsg.match(key="z", stimulus_id="s6"),
             Execute(key="x", stimulus_id="s6"),
         ]
@@ -813,11 +1037,11 @@ def test_workerstate_flight_failure_to_executing(ws, block_queue):
     assert ws.tasks["x"].state == "executing"
 
 
-def test_workerstate_resumed_fetch_to_executing(ws_with_running_task):
+def test_workerstate_resumed_fetch_to_cancelled_to_executing(ws_with_running_task):
     """Test state loops:
 
     - executing -> cancelled -> resumed(fetch) -> cancelled -> executing
-    - executing -> long-running -> cancelled -> resumed(fetch) -> long-running
+    - executing -> long-running -> cancelled -> resumed(fetch) -> cancelled -> long-running
 
     See also: test_workerstate_resumed_waiting_to_flight
     """
@@ -840,6 +1064,32 @@ def test_workerstate_resumed_fetch_to_executing(ws_with_running_task):
     assert ws.tasks["x"].state == prev_state
 
 
+def test_workerstate_resumed_fetch_to_executing(ws_with_running_task):
+    """See test_resumed_cancelled_handle_compute for end-to-end version"""
+    ws = ws_with_running_task
+    ws2 = "127.0.0.1:2"
+
+    prev_state = ws.tasks["x"].state
+
+    instructions = ws.handle_stimulus(
+        # x is released for whatever reason (e.g. client cancellation)
+        FreeKeysEvent(keys=["x"], stimulus_id="s1"),
+        # x was computed somewhere else
+        ComputeTaskEvent.dummy("y", who_has={"x": [ws2]}, stimulus_id="s2"),
+        # x was lost / no known replicas, therefore y is cancelled
+        FreeKeysEvent(keys=["y"], stimulus_id="s3"),
+        ComputeTaskEvent.dummy("x", stimulus_id="s4"),
+    )
+    if prev_state == "executing":
+        assert not instructions
+    else:
+        assert instructions == [
+            LongRunningMsg(key="x", compute_duration=None, stimulus_id="s4")
+        ]
+    assert len(ws.tasks) == 1
+    assert ws.tasks["x"].state == prev_state
+
+
 def test_workerstate_resumed_waiting_to_flight(ws):
     """Test state loop:
 
@@ -878,7 +1128,7 @@ async def test_execute_preamble_early_cancel(
     https://github.com/dask/dask/issues/9330
     test_worker.py::test_execute_preamble_abort_retirement
     """
-    async with BlockedExecute(s.address, validate=True) as a:
+    async with BlockedExecute(s.address) as a:
         if critical_section == "execute":
             in_ev = a.in_execute
             block_ev = a.block_execute
@@ -938,7 +1188,7 @@ def test_cancel_with_dependencies_in_memory(ws, release_dep, done_ev_cls):
 
     Read: https://github.com/dask/distributed/issues/6893"""
     ws.handle_stimulus(
-        UpdateDataEvent(data={"x": 1}, report=False, stimulus_id="s1"),
+        UpdateDataEvent(data={"x": 1}, stimulus_id="s1"),
         ComputeTaskEvent.dummy("y", who_has={"x": [ws.address]}, stimulus_id="s2"),
     )
     assert ws.tasks["x"].state == "memory"
@@ -1059,5 +1309,34 @@ async def test_secede_cancelled_or_resumed_scheduler(c, s, a):
     assert ws.processing
 
     await ev4.set()
-    assert await x == 123
+    assert await x == 2
     assert not ws.processing
+
+
+def test_workerstate_remove_replica_of_cancelled_task_dependency(ws):
+    """If a dependency was fetched, but the task gets freed by the scheduler
+    before the add-keys message arrives, the scheduler sends a remove-replica
+    message to the worker, which should then release the dependency.
+
+    Read: https://github.com/dask/distributed/pull/7487#issuecomment-1387277900
+
+        See test_resumed_cancelled_handle_compute for end-to-end version
+    """
+    ws2 = "127.0.0.1:2"
+    instructions = ws.handle_stimulus(
+        ComputeTaskEvent.dummy("y", who_has={"x": [ws2]}, stimulus_id="s1"),
+        GatherDepSuccessEvent(
+            worker=ws2, total_nbytes=1, data={"x": 123}, stimulus_id="s2"
+        ),
+        FreeKeysEvent(keys=["y"], stimulus_id="s3"),
+    )
+    assert ws.tasks["x"].state == "memory"
+    assert ws.tasks["y"].state == "cancelled"
+
+    # Test that the worker does accepts the RemoveReplicasEvent and
+    # subsequently releases the data
+    instructions = ws.handle_stimulus(
+        RemoveReplicasEvent(keys=["x"], stimulus_id="s4"),
+    )
+    assert ws.tasks["x"].state == "released"
+    assert instructions == [ReleaseWorkerDataMsg(stimulus_id="s4", key="x")]
