@@ -3,16 +3,20 @@ from __future__ import annotations
 import asyncio
 import contextlib
 import os
+import random
 import socket
+import sys
 import threading
 import time as timemod
 import weakref
+from unittest import mock
 
 import pytest
 from tornado.ioloop import IOLoop
 
 import dask
 
+from distributed.batched import BatchedSend
 from distributed.comm.core import CommClosedError
 from distributed.comm.registry import backends
 from distributed.comm.tcp import TCPBackend, TCPListener
@@ -33,7 +37,7 @@ from distributed.core import (
 from distributed.metrics import time
 from distributed.protocol import to_serialize
 from distributed.protocol.compression import compressions
-from distributed.utils import get_ip, get_ipv6
+from distributed.utils import get_ip, get_ipv6, wait_for
 from distributed.utils_test import (
     assert_can_connect,
     assert_can_connect_from_everywhere_4,
@@ -257,6 +261,16 @@ async def test_server_status_compare_enum_is_quiet():
     server.status == Status.running  # noqa: B015
 
 
+@gen_test(config={"distributed.admin.system-monitor.gil.enabled": True})
+async def test_server_close_stops_gil_monitoring():
+    pytest.importorskip("gilknocker")
+
+    server = Server({})
+    assert server.monitor._gilknocker.is_running
+    await server.close()
+    assert not server.monitor._gilknocker.is_running
+
+
 @gen_test()
 async def test_server():
     """
@@ -321,7 +335,7 @@ async def test_server_listen():
         if has_ipv6():
             EXTERNAL_IP6 = get_ipv6()
     except socket.gaierror:
-        raise pytest.skip("no network access")
+        pytest.skip("no network access")
 
     from contextlib import asynccontextmanager
 
@@ -695,7 +709,7 @@ async def test_send_recv_cancelled():
         server_comm = next(iter(server._comms))
 
         with pytest.raises(asyncio.TimeoutError):
-            await asyncio.wait_for(send_recv(client_comm, op="get_stuck"), timeout=0.1)
+            await wait_for(send_recv(client_comm, op="get_stuck"), timeout=0.1)
         assert client_comm.closed()
         while not server_comm.closed():
             await asyncio.sleep(0.01)
@@ -724,10 +738,12 @@ async def test_connection_pool():
         *(rpc(ip="127.0.0.1", port=s.port).ping() for s in servers[:5])
     )
     await asyncio.gather(*(rpc(s.address).ping() for s in servers[:5]))
-    await asyncio.gather(*(rpc("127.0.0.1:%d" % s.port).ping() for s in servers[:5]))
+    await asyncio.gather(*(rpc(f"127.0.0.1:{s.port}").ping() for s in servers[:5]))
     await asyncio.gather(
         *(rpc(ip="127.0.0.1", port=s.port).ping() for s in servers[:5])
     )
+    await asyncio.gather(*(rpc(("127.0.0.1", s.port)).ping() for s in servers[:5]))
+
     assert sum(map(len, rpc.available.values())) == 5
     assert sum(map(len, rpc.occupied.values())) == 0
     assert rpc.active == 0
@@ -754,6 +770,9 @@ async def test_connection_pool():
         assert time() < start + 2
 
     await rpc.close()
+    with pytest.raises(RuntimeError, match=r"ConnectionPool is closed"):
+        await rpc(ip="127.0.0.1", port=s.port).ping()
+
     await asyncio.gather(*[server.close() for server in servers])
 
 
@@ -840,8 +859,35 @@ async def test_connection_pool_outside_cancellation(monkeypatch):
 
 
 @gen_test()
-async def test_connection_pool_respects_limit():
+async def test_remove_cancels_connect_attempts():
+    loop = asyncio.get_running_loop()
+    connect_started = asyncio.Event()
+    connect_finished = loop.create_future()
 
+    async def connect(*args, **kwargs):
+        connect_started.set()
+        await connect_finished
+
+    async def connect_to_server():
+        with pytest.raises(CommClosedError, match="Address removed."):
+            await rpc.connect("tcp://0.0.0.0")
+
+    async def remove_address():
+        await connect_started.wait()
+        rpc.remove("tcp://0.0.0.0")
+
+    rpc = await ConnectionPool(limit=1)
+
+    with mock.patch("distributed.core.connect", connect):
+        await asyncio.gather(
+            connect_to_server(),
+            remove_address(),
+        )
+    assert connect_finished.cancelled()
+
+
+@gen_test()
+async def test_connection_pool_respects_limit():
     limit = 5
 
     async def ping(comm, delay=0.01):
@@ -1088,7 +1134,6 @@ async def test_close_properly():
     ip = get_ip()
     rpc_addr = f"tcp://{ip}:{ports[-1]}"
     async with rpc(rpc_addr) as remote:
-
         comm = await remote.live_comm()
         await comm.write({"op": "sleep"})
         await sleep_started.wait()
@@ -1201,10 +1246,32 @@ async def test_server_comms_mark_active_handlers():
             validate_dict(server)
 
 
+@gen_test()
+async def test_server_sys_path_local_directory_cleanup(tmp_path, monkeypatch):
+    local_directory = str(tmp_path / "dask-scratch-space")
+
+    # Ensure `local_directory` is removed from `sys.path` as part of the
+    # `Server` shutdown process
+    assert not any(i.startswith(local_directory) for i in sys.path)
+    async with Server({}, local_directory=local_directory):
+        assert sys.path[0].startswith(local_directory)
+    assert not any(i.startswith(local_directory) for i in sys.path)
+
+    # Ensure `local_directory` isn't removed from `sys.path` if it
+    # was already there before the `Server` started
+    monkeypatch.setattr("sys.path", [local_directory] + sys.path)
+    assert sys.path[0].startswith(local_directory)
+    # NOTE: `needs_workdir=False` is needed to make sure the same path added
+    # to `sys.path` above is used by the `Server` (a subdirectory is created
+    # by default).
+    async with Server({}, local_directory=local_directory, needs_workdir=False):
+        assert sys.path[0].startswith(local_directory)
+    assert sys.path[0].startswith(local_directory)
+
+
 @pytest.mark.parametrize("close_via_rpc", [True, False])
 @gen_test()
 async def test_close_fast_without_active_handlers(close_via_rpc):
-
     server = await Server({})
     server.handlers["terminate"] = server.close
     await server.listen(0)
@@ -1212,11 +1279,11 @@ async def test_close_fast_without_active_handlers(close_via_rpc):
 
     if not close_via_rpc:
         fut = server.close()
-        await asyncio.wait_for(fut, 0.5)
+        await wait_for(fut, 0.5)
     else:
         async with rpc(server.address) as _rpc:
             fut = _rpc.terminate(reply=False)
-            await asyncio.wait_for(fut, 0.5)
+            await wait_for(fut, 0.5)
 
 
 @gen_test()
@@ -1239,7 +1306,7 @@ async def test_close_grace_period_for_handlers():
     # since the handler is running for a while, the close will not immediately
     # go through. We'll give the comm about a second to close itself
     with pytest.raises(asyncio.TimeoutError):
-        await asyncio.wait_for(wait_for_close.wait(), 0.5)
+        await wait_for(wait_for_close.wait(), 0.5)
     await task
 
 
@@ -1315,3 +1382,78 @@ async def test_async_listener_stop(monkeypatch):
         async with Server({}) as s:
             await s.listen(0)
             assert s.listeners
+
+
+@gen_test()
+async def test_messages_are_ordered_bsend():
+    ledger = []
+
+    async def async_handler(val):
+        await asyncio.sleep(0.01 * random.random())
+        ledger.append(val)
+
+    def sync_handler(val):
+        ledger.append(val)
+
+    async with Server(
+        {},
+        stream_handlers={
+            "sync_handler": sync_handler,
+            "async_handler": async_handler,
+        },
+    ) as s:
+        await s.listen()
+        comm = await connect(s.address)
+        try:
+            b = BatchedSend(interval=10)
+            try:
+                await comm.write({"op": "connection_stream"})
+                b.start(comm)
+                n = 100
+                for ix in range(n):
+                    if ix % 2:
+                        b.send({"op": "sync_handler", "val": ix})
+                    else:
+                        b.send({"op": "async_handler", "val": ix})
+                while not len(ledger) == n:
+                    await asyncio.sleep(0.01)
+                assert ledger == list(range(n))
+            finally:
+                await b.close()
+        finally:
+            await comm.close()
+
+
+@gen_test()
+async def test_messages_are_ordered_raw():
+    ledger = []
+
+    async def async_handler(val):
+        await asyncio.sleep(0.01 * random.random())
+        ledger.append(val)
+
+    def sync_handler(val):
+        ledger.append(val)
+
+    async with Server(
+        {},
+        stream_handlers={
+            "sync_handler": sync_handler,
+            "async_handler": async_handler,
+        },
+    ) as s:
+        await s.listen()
+        comm = await connect(s.address)
+        try:
+            await comm.write({"op": "connection_stream"})
+            n = 100
+            for ix in range(n):
+                if ix % 2:
+                    await comm.write({"op": "sync_handler", "val": ix})
+                else:
+                    await comm.write({"op": "async_handler", "val": ix})
+            while not len(ledger) == n:
+                await asyncio.sleep(0.01)
+            assert ledger == list(range(n))
+        finally:
+            await comm.close()
